package com.winhc.task.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.winhc.task.util.BaseUtils;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.val;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiFunction;

import static com.winhc.task.util.BaseUtils.*;

import com.winhc.task.util.PreUDF;

/**
 * 配置维度信息
 *
 * @author ZhangJi
 * @since 2021-10-22 15:12
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
public class ArgsCompanyJob {
    private MetaInfo.HbaseInfo hbaseInfo;
    private MetaInfo.ElasticSearchInfo esInfo;
    private MetaInfo.HologresInfo holoInfo;
    private String[] md5Fields;
    private BiFunction<String, JSONObject, String> udf;
    private boolean useCompanyId;
    public static final Map<String, ArgsCompanyJob> JOB_ARGS = createArgs();

    public ArgsCompanyJob(MetaInfo.HbaseInfo hbaseInfo, MetaInfo.ElasticSearchInfo esInfo, MetaInfo.HologresInfo holoInfo, String[] md5Fields, BiFunction<String, JSONObject, String> udf) {
        this.hbaseInfo = hbaseInfo;
        this.esInfo = esInfo;
        this.holoInfo = holoInfo;
        this.md5Fields = md5Fields;
        this.useCompanyId = true;
        this.udf = udf;
    }

    public static Map<String, ArgsCompanyJob> createArgs() {
        HashMap<String, ArgsCompanyJob> a = new HashMap<>();
        a.put("company", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company", "public"),
                null,
                (s, jsonObject) -> s
        ));
        a.put("company_holder", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_HOLDER", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_holder", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_holder", "public"),
                null,
                (companyId, j) -> {
                    val row = PreUDF.company_holder_rowkey(j.getInteger("holder_type"), j.getString("holder_id"), j.getString("holder_name"));
                    return concatws("_", companyId, md5(BaseUtils.cleanup(row)));
                }
        ));
        a.put("company_holder_sponsor", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_HOLDER_SPONSOR", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_holder_sponsor", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_holder_sponsor", "public"),
                null,
                (companyId, j) -> {
                    val row = PreUDF.company_holder_rowkey(j.getInteger("holder_type"), j.getString("holder_id"), j.getString("holder_name"));
                    return concatws("_", companyId, md5(BaseUtils.cleanup(row)));
                }
        ));
        a.put("company_staff", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_STAFF", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_staff", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_staff", "public"),
                new String[]{"staff_name"},
                null
        ));
        a.put("company_change", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_CHANGE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_change", "_doc"),
                new MetaInfo.HologresInfo("", ""),
                null,
                ArgsCompanyJob::getChangeRowkey

        ));
        a.put("company_finance", new ArgsCompanyJob(
                        new MetaInfo.HbaseInfo("NG_RT_COMPANY_FINANCE", "F"),
                        new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_finance", "_doc"),
                        new MetaInfo.HologresInfo("ng_rt_company_finance", "public"),
                        null,
                        (companyId, j) -> {
                            String round = j.getString("round");
                            return md5(cleanup(concatws("", companyId, round)));
                        }
                )
        );
        a.put("company_icp", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ICP", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_icp", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_icp", "public"),
                new String[]{"domain", "web_baxh", "company_name"},
                null,
                false
        ));
        a.put("company_app_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_APP_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_app_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_app_info", "public"),
                new String[]{"name"},
                null
        ));
        a.put("company_tm", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_TM", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_tm", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_tm", "public"),
                new String[]{"reg_no", "int_cls"},
                null
        ));

        a.put("company_dishonest_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_DISHONEST_INFO", "F", j -> {
                    j.remove("is_filter");
                    return j;
                }),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_dishonest_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_dishonest_info", "public"),
                new String[]{"case_no", "name"},
                null
        ));

        a.put("zxr_evaluate_results", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_ZXR_EVALUATE_RESULTS", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_zxr_evaluate_results", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_zxr_evaluate_results", "public"),
                new String[]{"case_no", "asset_name", "name", "publish_time"},
                null
        ));
        a.put("zxr_evaluate", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_ZXR_EVALUATE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_zxr_evaluate", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_zxr_evaluate", "public"),
                new String[]{"case_no", "asset_name", "name"},
                null
        ));
        a.put("company_zxr_restrict", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ZXR_RESTRICT", "F"
                        , j -> {
                    j.remove("is_filter");
                    return j;
                }),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_zxr_restrict", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_zxr_restrict", "public"),
                new String[]{"case_no", "person_name", "company_name"},
                null,
                false
        ));
        a.put("company_zxr_final_case", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ZXR_FINAL_CASE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_zxr_final_case", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_zxr_final_case", "public"),
                new String[]{"case_no", "name"},
                null
        ));
        a.put("company_zxr", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ZXR", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_zxr", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_zxr", "public"),
                new String[]{"name", "case_no"},
                null
        ));
        a.put("bankruptcy_judgment_document", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_BANKRUPTCY_JUDGMENT_DOCUMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_bankruptcy_judgment_document", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_bankruptcy_judgment_document", "public"),
                null,
                (companyId, j) -> {
                    String title = j.getString("title");
                    String case_no = j.getString("case_no");
                    String main_id = j.getString("main_id");
                    String public_date = splitDate(j.getString("public_date"));
                    return concatws("_", main_id, md5(cleanup(concatws("", title, case_no, public_date))));
                }
        ));
        a.put("bankruptcy_open_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_BANKRUPTCY_OPEN_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_bankruptcy_open_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_bankruptcy_open_announcement", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String title = j.getString("title");
                    String case_no = j.getString("case_no");
                    String public_date = splitDate(j.getString("public_date"));
                    return concatws("_", main_id, md5(cleanup(concatws("", title, case_no, public_date))));
                }
        ));
        a.put("bankruptcy_open_case", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_BANKRUPTCY_OPEN_CASE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_bankruptcy_open_case", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_bankruptcy_open_case", "public"),
                null,
                (companyId, j) -> {
                    String applicant = j.getString("applicant");
                    String respondent = j.getString("respondent");
                    String case_no = j.getString("case_no");
                    String public_date = j.getString("public_date");
                    return md5(cleanup(concatws("", applicant, respondent, case_no, splitDate(public_date))));
                }
        ));
        a.put("company_court_open_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_COURT_OPEN_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_court_open_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_court_open_announcement", "public"),
                null,
                (companyId, j) -> {
                    String case_no = j.getString("case_no");
                    if (StringUtils.isEmpty(case_no)) {
                        return BaseUtils.getCompanyCourtOpenAnnouncementRowkey(companyId, j);
                    }
                    String start_date = splitDate(j.getString("start_date"));
                    return md5(cleanup(concatws("", case_no, start_date)));
                }
        ));
        a.put("company_court_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_COURT_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_court_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_court_announcement", "public"),
                null,
                (companyId, j) -> {
                    String case_no = j.getString("case_no");
                    String announcement_type = j.getString("announcement_type");
                    String publish_date = j.getString("publish_date");
                    String court_name = j.getString("court_name");
                    String plaintiff = j.getString("plaintiff");
                    String litigant = j.getString("litigant");
                    //String bltn_no = j.getString("bltn_no");
                    return md5(cleanup(concatws("", case_no, announcement_type, splitDate(publish_date), court_name, plaintiff, litigant/*, bltn_no*/)));
                }
        ));
        a.put("company_court_register", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_COURT_REGISTER", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_court_register", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_court_register", "public"),
                null,
                (companyId, j) -> {
                    String case_no = j.getString("case_no");
                    String filing_date = j.getString("filing_date");
                    return md5(cleanup(concatws("", case_no, splitDate(filing_date))));
                }
        ));
        a.put("company_send_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_SEND_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_send_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_send_announcement", "public"),
                null,
                (companyId, j) -> {
                    String court = j.getString("court");
                    String start_date = j.getString("start_date");
                    String title = j.getString("title");
                    String case_no = j.getString("case_no");
                    return md5(cleanup(concatws("", court, splitDate(start_date), title, case_no)));
                }
        ));
        a.put("litigation_mediation", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_LITIGATION_MEDIATION", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_litigation_mediation", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_litigation_mediation", "public"),
                null,
                (companyId, j) -> {
                    String case_no = j.getString("case_no");
                    String filing_date = j.getString("filing_date");
                    return md5(cleanup(concatws("", case_no, splitDate(filing_date))));
                }
        ));
        a.put("restrictions_on_exit", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_RESTRICTIONS_ON_EXIT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_restrictions_on_exit", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_restrictions_on_exit", "public"),
                null,
                (companyId, j) -> {
                    String case_no = j.getString("case_no");
                    String limited_person = j.getString("limited_person");
                    String executed_person_name = j.getString("executed_person_name");
                    String publish_time = j.getString("publish_time");
                    return md5(cleanup(concatws("", case_no, limited_person, executed_person_name, publish_time)));
                }

        ));
        a.put("auction_tracking", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_AUCTION_TRACKING", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_auction_tracking", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_auction_tracking", "public"),
                null,
                (companyId, j) -> j.getString("id")
        ));
        a.put("property_rights_transaction", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_PROPERTY_RIGHTS_TRANSACTION", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_property_rights_transaction", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_property_rights_transaction", "public"),
                new String[]{"pro_no", "pro_name"},
                null
        ));
        a.put("company_equity_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_EQUITY_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_equity_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_equity_info", "public"),
                null,
                (companyId, j) -> {
                    String reg_number = j.getString("reg_number");
                    String related_company_name = j.getString("related_company_name");
                    JSONArray pledgee_info = j.getJSONArray("pledgee_info");
                    JSONArray pledgor_info = j.getJSONArray("pledgor_info");
                    return md5(cleanup(concatws("", reg_number, equity_info_rowkey(pledgee_info, pledgor_info), related_company_name)));
                }
        ));
        a.put("company_equity_pledge", new ArgsCompanyJob(
                null,
                new MetaInfo.ElasticSearchInfo("winhc_index_rt_company_equity_pledge", "_doc", 6),
                new MetaInfo.HologresInfo("ng_rt_company_equity_pledge", "public"),
                null,
                (companyId, j) -> {
                    String stock_code = j.getString("stock_code");
                    String transaction_date = j.getString("transaction_date");
                    String pledged_num = j.getString("pledged_num");
                    return md5(cleanup(concatws("", stock_code, splitDate(transaction_date), pledged_num)));
                }
        ));
        a.put("company_equity_pledge_holder", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_EQUITY_PLEDGE_HOLDER", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_index_rt_company_equity_pledge_holder", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_equity_pledge_holder", "public"),
                null,
                (companyId, j) -> {
                    String scode = j.getString("scode");
                    String holder_name = j.getString("holder_name");
                    String pledgee = j.getString("pledgee");
                    String sharefrozennum = j.getString("sharefrozennum");
                    String ndate = j.getString("ndate");
                    String sdate = j.getString("sdate");
                    return md5(cleanup(concatws("", scode, holder_name, pledgee, sharefrozennum, splitDate(ndate), splitDate(sdate))));
                }
        ));
        a.put("company_abnormal_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ABNORMAL_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_abnormal_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_abnormal_info", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String put_reason = j.getString("put_reason");
                    String put_date = j.getString("put_date");
                    return concatws("_", company_id, md5(cleanup(concatws("", put_reason, splitDate(put_date)))));
                }
        ));
        a.put("company_illegal_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ILLEGAL_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_illegal_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_illegal_info", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String put_reason = j.getString("put_reason");
                    String put_date = j.getString("put_date");
                    return concatws("_", company_id, md5(cleanup(concatws("", put_reason, splitDate(put_date)))));
                }
        ));
        a.put("company_judicial_assistance", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_JUDICIAL_ASSISTANCE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_judicial_assistance", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_judicial_assistance", "public"),
                null,
                (companyId, j) -> {
                    String executed_person = j.getString("executed_person");
                    String execute_notice_num = j.getString("execute_notice_num");
                    //String type_state = j.getString("type_state");
                    String company_id = j.getString("company_id");
                    return md5(cleanup(concatws("", executed_person, execute_notice_num, /*type_state,*/ company_id)));
                }
        ));
        a.put("company_brief_cancel_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_BRIEF_CANCEL_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_brief_cancel_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_brief_cancel_announcement", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String announcement_end_date = j.getString("announcement_end_date");
                    return md5(cleanup(concatws("", company_id, splitDate(announcement_end_date))));
                }
        ));
        a.put("company_brief_cancel_announcement_objection", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_BRIEF_CANCEL_ANNOUNCEMENT_OBJECTION", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_brief_cancel_announcement_objection", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_brief_cancel_announcement_objection", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String objection_date = j.getString("objection_date");
                    String objection_content = j.getString("objection_content");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, splitDate(objection_date), objection_content))));
                }
        ));
        a.put("company_brief_cancel_announcement_result", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_BRIEF_CANCEL_ANNOUNCEMENT_RESULT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_brief_cancel_announcement_result", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_brief_cancel_announcement_result", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String announcement_apply_date = j.getString("announcement_apply_date");
                    String brief_cancel_result = j.getString("brief_cancel_result");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, splitDate(announcement_apply_date), brief_cancel_result))));
                }
        ));
        a.put("high_tech_enterprise", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_HIGH_TECH_ENTERPRISE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_index_rt_high_tech_enterprise", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_high_tech_enterprise", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String certificate_num = j.getString("certificate_num");
                    return md5(cleanup(concatws("", company_id, certificate_num)));
                }
        ));
        a.put("company_liquidating_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LIQUIDATING_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_liquidating_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_liquidating_info", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    return md5(cleanup(company_id));
                }
        ));
        a.put("company_license", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LICENSE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_license", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_license", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String license_number = j.getString("license_number");
                    return md5(cleanup(concatws("", company_id, license_number)));
                }
        ));
        a.put("company_punishment_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_PUNISHMENT_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_punishment_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_punishment_info", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String punish_number = j.getString("punish_number");
                    return md5(cleanup(concatws("", company_id, punish_number)));
                }
        ));
        a.put("company_check_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_CHECK_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_check_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_check_info", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String check_type = j.getString("check_type");
                    String check_result = j.getString("check_result");
                    String check_org = j.getString("check_org");
                    String check_date = j.getString("check_date");
                    return md5(cleanup(concatws("", check_type, check_result, check_org, splitDate(check_date), company_id)));
                }
        ));
        a.put("company_double_random_check_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_DOUBLE_RANDOM_CHECK_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_double_random_check_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_double_random_check_info", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String check_task_num = j.getString("check_task_num");
                    String check_plan_num = j.getString("check_plan_num");
                    return md5(cleanup(concatws("", check_task_num, check_plan_num, company_id)));
                }
        ));
        a.put("company_double_random_check_result_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_DOUBLE_RANDOM_CHECK_RESULT_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_double_random_check_result_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_double_random_check_result_info", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String check_item = j.getString("check_item");
                    String check_result = j.getString("check_result");
                    return concatws("_", main_id, md5(cleanup(concatws("", check_item, check_result))));
                }
        ));
        a.put("cancellation_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_CANCELLATION_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_cancellation_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_cancellation_announcement", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    return md5(cleanup(concatws("", company_id)));
                }
        ));
        a.put("company_phone", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_PHONE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_phone", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_phone", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String phone = j.getString("phone");
                    return md5(cleanup(concatws("", company_id, phone)));
                }
        ));
        a.put("online_retailers", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_ONLINE_RETAILERS", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_online_retailers", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_online_retailers", "public"),
                null,
                (companyId, j) -> {
                    String shop_url = getHost(j.getString("shop_url"));
                    if (shop_url == null) return null;
                    return md5(shop_url);
                }
        ));
        a.put("company_annual_report", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String report_year = j.getString("report_year");
                    return md5(cleanup(concatws("", company_id, report_year)));
                }
        ));
        a.put("company_annual_report_out_investment", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_OUT_INVESTMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_out_investment", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_out_investment", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String out_investment_name = j.getString("out_investment_name");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, out_investment_name))));
                }
        ));
        a.put("company_annual_report_out_guarantee", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_OUT_GUARANTEE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_out_guarantee", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_out_guarantee", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String creditor = j.getString("creditor");
                    String obligor = j.getString("obligor");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, creditor, obligor))));
                }
        ));
        a.put("company_annual_report_change", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_CHANGE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_change", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_change", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String change_item = j.getString("change_item");
                    String change_time = j.getString("change_time");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, change_item, splitDate(change_time)))));
                }
        ));
        a.put("company_annual_report_equity_change", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_EQUITY_CHANGE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_equity_change", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_equity_change", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String change_time = j.getString("change_time");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, splitDate(change_time)))));
                }
        ));
        a.put("company_annual_report_holder", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_HOLDER", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_holder", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_holder", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String investor_name = j.getString("investor_name");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, investor_name))));
                }
        ));
        a.put("company_annual_report_webinfo", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_WEBINFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_webinfo", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_webinfo", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    String website = j.getString("website");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id, website))));
                }
        ));
        a.put("company_annual_report_social_security", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ANNUAL_REPORT_SOCIAL_SECURITY", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_annual_report_social_security", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_annual_report_social_security", "public"),
                null,
                (companyId, j) -> {
                    String main_id = j.getString("main_id");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id))));
                }
        ));
        a.put("finance_info_v3", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_FINANCE_INFO_V3", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_finance_info_v3", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_finance_info_v3", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String year = j.getString("year");
                    return md5(cleanup(concatws("", company_id, year)));
                }
        ));
        a.put("private_enterprise", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_PRIVATE_ENTERPRISE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_private_enterprise", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_private_enterprise", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String year = j.getString("year");
                    return md5(cleanup(concatws("", company_id, year)));
                }
        ));
        a.put("company_tags", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_TAGS", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_tags", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_tags", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String tag_code = j.getString("tag_code");
                    return concatws("_", company_id, tag_code);
                }
        ));
        a.put("construction_person", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_CONSTRUCTION_PERSON", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_construction_person", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_construction_person", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    String person_name = j.getString("person_name");
                    String zczy_name = j.getString("zczy_name");
                    String seal_code = j.getString("seal_code");
                    return md5(cleanup(concatws("", company_id, person_name, zczy_name, seal_code)));
                }
        ));
        a.put("construction_project", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_CONSTRUCTION_PROJECT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_construction_project", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_construction_project", "public"),
                null,
                (companyId, j) -> {
                    //"company_name", "prj_name", "prj_num"
                    String company_name = j.getString("company_name");
                    String prj_name = j.getString("prj_name");
                    String prj_num = j.getString("prj_num");
                    return md5(cleanup(concatws("", company_name, prj_name, prj_num)));
                }
        ));
        a.put("construction_qualification", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_CONSTRUCTION_QUALIFICATION", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_construction_qualification", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_construction_qualification", "public"),
                null,
                (companyId, j) -> {
//"company_id", "cert_type", "cert_id", "cert_name"
                    String company_id = j.getString("company_id");
                    String cert_type = j.getString("cert_type");
                    String cert_id = j.getString("cert_id");
                    String cert_name = j.getString("cert_name");
                    return md5(cleanup(concatws("", company_id, cert_type, cert_id, cert_name)));
                }
        ));
        a.put("company_bid_new", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_BID_NEW", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_bid_new", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_bid_new", "public"),
                null,
                (companyId, j) -> {
                    //"title", "link", "publish_time"
                    String title = j.getString("title");
                    String source_url = j.getString("source_url");
                    String publish_time = j.getString("publish_time");
                    return md5(cleanup(concatws("", title, source_url, splitDate(publish_time))));
                }
        ));
        a.put("reduction_registered_capital_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_REDUCTION_REGISTERED_CAPITAL_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_reduction_registered_capital_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_reduction_registered_capital_info", "public"),
                null,
                ArgsCompanyJob::getChangeRowkey
        ));
        a.put("increase_registered_capital_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_INCREASE_REGISTERED_CAPITAL_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_increase_registered_capital_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_increase_registered_capital_info", "public"),
                null,
                ArgsCompanyJob::getChangeRowkey
        ));
        a.put("general_taxpayer", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_GENERAL_TAXPAYER", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_general_taxpayer", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_general_taxpayer", "public"),
                null,
                (companyId, j) -> {
                    //"company_id"
                    String company_id = j.getString("company_id");
                    return md5(cleanup(concatws("", company_id)));
                }
        ));
        a.put("company_mortgage_info", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_MORTGAGE_INFO", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_mortgage_info", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_mortgage_info", "public"),
                null,
                (companyId, j) -> {
                    //"reg_num"
                    String reg_num = j.getString("reg_num");
                    return md5(cleanup(concatws("", reg_num)));
                }
        ));
        a.put("company_mortgage_pawn", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_MORTGAGE_PAWN", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_mortgage_pawn", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_mortgage_pawn", "public"),
                null,
                (companyId, j) -> {
                    //"main_id", "pawn_name", "detail"
                    String main_id = j.getString("main_id");
                    String pawn_name = j.getString("pawn_name");
                    String detail = j.getString("detail");
                    return concatws("_", main_id, md5(cleanup(concatws("", pawn_name, detail))));
                }
        ));
        a.put("company_mortgage_change", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_MORTGAGE_CHANGE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_mortgage_change", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_mortgage_change", "public"),
                null,
                (companyId, j) -> {
                    //"main_id", "change_date", "change_content"
                    String main_id = j.getString("main_id");
                    String change_date = j.getString("change_date");
                    String change_content = j.getString("change_content");

                    return concatws("_", main_id, md5(cleanup(concatws("", splitDate(change_date), change_content))));
                }
        ));
        a.put("company_mortgage_people", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_MORTGAGE_PEOPLE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_mortgage_people", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_mortgage_people", "public"),
                null,
                (companyId, j) -> {
                    //"main_id"
                    String main_id = j.getString("main_id");
                    return concatws("_", main_id, md5(cleanup(concatws("", main_id))));
                }
        ));
        a.put("wenshu_detail_v2", new ArgsCompanyJob(
                null,
                //new MetaInfo.ElasticSearchInfo("wenshu_detail5", "wenshu_detail_type", 5),
                new MetaInfo.ElasticSearchInfo("wenshu_detail2", "wenshu_detail_type", 5),
                new MetaInfo.HologresInfo("ng_rt_wenshu_detail_v2", "public"),
                null,
                (companyId, j) -> {
                    String case_no = j.getString("case_no");
                    String title = j.getString("title");
                    return md5(cleanup(concatws("", case_no, title)));
                }
        ));

        a.put("company_env_punishment", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_ENV_PUNISHMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_env_punishment", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_env_punishment", "public"),
                null,
                (companyId, j) -> {
                    //"name", "punish_number"
                    String name = j.getString("name");
                    String punish_number = j.getString("punish_number");
                    return md5(cleanup(concatws("", name, punish_number)));

                }

        ));
        a.put("company_own_tax", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_OWN_TAX", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_own_tax", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_own_tax", "public"),
                null,
                (companyId, j) -> {
                    //"tax_balance", "tax_category", "company_id"
                    String tax_balance = j.getString("tax_balance");
                    String tax_category = j.getString("tax_category");
                    String company_id = j.getString("company_id");
                    return md5(cleanup(concatws("", tax_balance, tax_category, company_id)));

                }));
        a.put("company_tax_contravention", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_TAX_CONTRAVENTION", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_tax_contravention", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_tax_contravention", "public"),
                null,
                (companyId, j) -> {
                    //"case_info", "company_id"
                    String case_info = j.getString("case_info");
                    String company_id = j.getString("company_id");
                    return md5(cleanup(concatws("", case_info, company_id)));
                }
        ));
        a.put("company_punishment_info_creditchina", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_PUNISHMENT_INFO_CREDITCHINA", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_punishment_info_creditchina", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_punishment_info_creditchina", "public"),
                null,
                (companyId, j) -> {
                    //"company_name", "punish_number"
                    String company_name = j.getString("company_name");
                    String punish_number = j.getString("punish_number");
                    return md5(cleanup(concatws("", company_name, punish_number)));
                }
        ));
        a.put("company_license_creditchina", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LICENSE_CREDITCHINA", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_license_creditchina", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_license_creditchina", "public"),
                null,
                (companyId, j) -> {
                    //"company_name", "license_number"
                    String company_name = j.getString("company_name");
                    String license_number = j.getString("license_number");
                    return md5(cleanup(concatws("", company_name, license_number)));
                }
        ));
        a.put("company_license_entpub", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LICENSE_ENTPUB", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_license_entpub", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_license_entpub", "public"),
                null,
                (companyId, j) -> {
                    //"company_name", "license_number"
                    String company_name = j.getString("company_name");
                    String license_number = j.getString("license_number");
                    return md5(cleanup(concatws("", company_name, license_number)));
                }
        ));
        a.put("company_public_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_PUBLIC_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_public_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_public_announcement", "public"),
                new String[]{"bill_num"},
                null,
                false
        ));
        a.put("company_land_announcement", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LAND_ANNOUNCEMENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_land_announcement", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_land_announcement", "public"),
                null,
                (companyId, j) -> {
                    //"e_number", "project_name"
                    String e_number = j.getString("e_number");
                    String project_name = j.getString("project_name");
                    return md5(cleanup(concatws("", e_number, project_name)));
                }
        ));
        a.put("company_land_publicity", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LAND_PUBLICITY", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_land_publicity", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_land_publicity", "public"),
                null,
                (companyId, j) -> {
                    //"title", "project_name"
                    String title = j.getString("title");
                    String project_name = j.getString("project_name");
                    return md5(cleanup(concatws("", title, project_name)));
                }
        ));
        a.put("company_land_transfer", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LAND_TRANSFER", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_land_transfer", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_land_transfer", "public"),
                null,
                (companyId, j) -> {
                    //"num", "location"
                    String num = j.getString("num");
                    String location = j.getString("location");
                    return md5(cleanup(concatws("", num, location)));
                }
        ));
        a.put("company_land_mortgage", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_LAND_MORTGAGE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_land_mortgage", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_land_mortgage", "public"),
                new String[]{"land_mark", "land_num", "other_item_num", "use_right_num"},
                null
        ));
        a.put("company_tax", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_TAX", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_tax", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_tax", "public"),
                null,
                (companyId, j) -> {
                    String company_name = j.getString("company_name");
                    String year = j.getString("year");
                    return md5(cleanup(concatws("", company_name, year)));
                }
        ));
        a.put("company_patent", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_PATENT", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_index_company_patent", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_patent", "public"),
                null,
                (companyId, j) -> {
                    String app_number = substr(j.getString("app_number"), 3);
                    String pub_number = substr(j.getString("pub_number"), 3);
                    String applicant_name = get_text_from_json(j.getString("applicant_name_info"), "name");
                    return md5(cleanup(concatws("", app_number, pub_number, applicant_name)));
                }
        ));
        a.put("company_copyright_reg", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_COPYRIGHT_REG", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_company_copyright_reg", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_copyright_reg", "public"),
                null,
                (companyId, j) -> {
                    String reg_num = j.getString("reg_num");
                    String author_nationality = get_text_from_json(j.getString("author_nationality_info"), "name");
                    return md5(cleanup(concatws("", reg_num, author_nationality)));
                }
        ));
        a.put("company_copyright_works", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_COPYRIGHT_WORKS", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_rt_company_copyright_works", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_copyright_works", "public"),
                null,
                (companyId, j) -> {
                    String reg_num = j.getString("reg_num");
                    String author = get_text_from_json(j.getString("author_info"), "name");
                    return md5(cleanup(concatws("", reg_num, author)));
                }
        ));
        a.put("legislation_info", new ArgsCompanyJob(
                null,
                new MetaInfo.ElasticSearchInfo("legislation_info_fx_v2", "legislation_info_fx_type", 5),
                new MetaInfo.HologresInfo("ng_rt_legislation_info", "public"),
                null,
                (companyId, j) -> j.getString("id")
        ));
        a.put("company_profile", new ArgsCompanyJob(
                new MetaInfo.HbaseInfo("NG_RT_COMPANY_PROFILE", "F"),
                new MetaInfo.ElasticSearchInfo("winhc_index_rt_company_profile", "_doc"),
                new MetaInfo.HologresInfo("ng_rt_company_profile", "public"),
                null,
                (companyId, j) -> {
                    String company_id = j.getString("company_id");
                    return md5(cleanup(concatws("", company_id)));
                }
        ));
        return a;
    }

    private static String getChangeRowkey(String companyId, JSONObject j) {
        String content_before = cleanupChange(j.getString("content_before"));
        String content_after = cleanupChange(j.getString("content_after"));
        String change_time = splitDate(j.getString("change_time"));
        if (StringUtils.isEmpty(content_before) && StringUtils.isEmpty(content_after)) {
            return null;
        }
        return concatws("_", companyId, md5(concatws("", change_time, content_before, content_after)));
    }

    public static String getHost(String url) {
        if (url == null || url.length() == 0)
            return null;

        int doubleslash = url.indexOf("//");
        if (doubleslash == -1)
            return null;
        else
            doubleslash += 2;
        int end = url.length();
        int port = url.indexOf(':', doubleslash);
        end = (port > 0 && port < end) ? port : end;
        return url.substring(doubleslash, end);

    }

    public static void main(String[] args) {
        System.out.println(getHost("http://www.baidu.com/a.html"));
        System.out.println(getHost("https://www.baidu.com/aa/"));
        System.out.println(getHost("https://mall.jd.com/index-1000286150.html"));
    }
}
