package com.winhc.task.common;

import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.winhc.task.util.BaseUtils.*;

/**
 * @author π
 * @since 2021-11-1 11:13
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
public class SummaryArgs {
    private String tableName;
    private List<ArgsInfo> argsInfo;
    private List<ValueAlias> valueAlias;
    private Function<JSONObject, List<String>> handles;
    public static final Map<String, SummaryArgs> SUMMARY_ARGS = createArgs();

    private static Map<String, SummaryArgs> createArgs() {
        HashMap<String, SummaryArgs> a = new HashMap<>();
        a.put("company_branch", new SummaryArgs(
                "ng_rt_company_branch"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_branch_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_branch_del_1")
        )
                , (j) -> Collections.singletonList(j.getString("parent_company_id"))
        ));
        a.put("company_holder", new SummaryArgs(
                "ng_rt_company_holder"
                , Arrays.asList(new ArgsInfo(
                "company_id"
                , "company_id"
                , "deleted"), new ArgsInfo(
                "invest"
                , "invest"
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        "company_id"
                        , "0"
                        , "company_holder_del_0"),
                new ValueAlias(
                        "company_id"
                        , "1"
                        , "company_holder_del_1")
                , new ValueAlias(
                        "invest"
                        , "0"
                        , "company_holder_del_0_invest"),
                new ValueAlias(
                        "invest"
                        , "1"
                        , "company_holder_del_1_invest")
        )
                , (j) -> {
            //Collections.singletonList(j.getString("company_id"))
            //暂时只计算人摘要
            return Stream.of(getKey(j, "company_id"), getKey(j, "holder_id"))
                    .filter(x -> x.length() == 32).distinct().collect(Collectors.toList());
        }
        ));
        a.put("company_holder_sponsor", new SummaryArgs(
                "ng_rt_company_holder_sponsor"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_holder_sponsor_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_holder_sponsor_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_staff", new SummaryArgs(
                "ng_rt_company_staff"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_staff_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_staff_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_change", new SummaryArgs(
                "ng_rt_company_change"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_change_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_change_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_tm", new SummaryArgs(
                "ng_rt_company_tm"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_tm_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_tm_del_1"))
                , (j) -> Collections.singletonList(j.getString("keyno"))
        ));
        a.put("company_app_info", new SummaryArgs(
                "ng_rt_company_app_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_app_info_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_app_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_icp", new SummaryArgs(
                "ng_rt_company_icp"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_icp_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_icp_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_finance", new SummaryArgs(
                "ng_rt_company_finance"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_finance_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_finance_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_dishonest_info", new SummaryArgs(
                "ng_rt_company_dishonest_info"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_dishonest_info_del_0"
                        , true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_dishonest_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("keyno"))
        ));
        a.put("company_zxr", new SummaryArgs(
                "ng_rt_company_zxr"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_zxr_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_zxr_del_1"))
                , (j) -> Collections.singletonList(j.getString("keyno"))
        ));
        a.put("company_zxr_restrict", new SummaryArgs(
                "ng_rt_company_zxr_restrict"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_zxr_restrict_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_zxr_restrict_del_1"))
                , (j) -> Stream.of(getKey(j, "pid"), getKey(j, "company_id")).distinct().collect(Collectors.toList())
        ));
        a.put("company_zxr_final_case", new SummaryArgs(
                "ng_rt_company_zxr_final_case"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_zxr_final_case_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_zxr_final_case_del_1"))
                , (j) -> Collections.singletonList(j.getString("keyno"))
        ));

        a.put("zxr_evaluate", new SummaryArgs(
                "ng_rt_zxr_evaluate"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "zxr_evaluate_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "zxr_evaluate_del_1"))
                , (j) -> Collections.singletonList(j.getString("keyno"))
        ));
        a.put("zxr_evaluate_results", new SummaryArgs(
                "ng_rt_zxr_evaluate_results"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "zxr_evaluate_results_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "zxr_evaluate_results_del_1"))
                , (j) -> Collections.singletonList(j.getString("keyno"))
        ));
        a.put("bankruptcy_open_case", new SummaryArgs(
                "ng_rt_bankruptcy_open_case"
                , Arrays.asList(new ArgsInfo(
                "applicant"
                , "applicant"
                , "deleted"), new ArgsInfo(
                "respondent"
                , "respondent"
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        "applicant"
                        , "0"
                        , "bankruptcy_open_case_del_0_applicant"),
                new ValueAlias(
                        "applicant"
                        , "1"
                        , "bankruptcy_open_case_del_1_applicant"), new ValueAlias(
                        "respondent"
                        , "0"
                        , "bankruptcy_open_case_del_0_respondent", true),
                new ValueAlias(
                        "respondent"
                        , "1"
                        , "bankruptcy_open_case_del_1_respondent"))
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("applicant_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("respondent_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_court_open_announcement", new SummaryArgs(
                "ng_rt_company_court_open_announcement"
                , Arrays.asList(new ArgsInfo("plaintiff"
                        , "plaintiff"
                        , "deleted")
                , new ArgsInfo("defendant"
                        , "defendant"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("plaintiff"
                        , "0"
                        , "company_court_open_announcement_del_0_plaintiff")
                , new ValueAlias("plaintiff"
                        , "1"
                        , "company_court_open_announcement_del_1_plaintiff")
                , new ValueAlias("defendant"
                        , "0"
                        , "company_court_open_announcement_del_0_defendant")
                , new ValueAlias("defendant"
                        , "1"
                        , "company_court_open_announcement_del_1_defendant")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("plaintiff_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("defendant_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_court_announcement", new SummaryArgs(
                "ng_rt_company_court_announcement"
                , Arrays.asList(new ArgsInfo("litigant"
                        , "litigant"
                        , "deleted")
                , new ArgsInfo("plaintiff"
                        , "plaintiff"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("litigant"
                        , "0"
                        , "company_court_announcement_del_0_litigant")
                , new ValueAlias("litigant"
                        , "1"
                        , "company_court_announcement_del_1_litigant")
                , new ValueAlias("plaintiff"
                        , "0"
                        , "company_court_announcement_del_0_plaintiff")
                , new ValueAlias("plaintiff"
                        , "1"
                        , "company_court_announcement_del_1_plaintiff")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("litigant_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("plaintiff_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_send_announcement", new SummaryArgs(
                "ng_rt_company_send_announcement"
                , Arrays.asList(new ArgsInfo("plaintiff"
                        , "plaintiff"
                        , "deleted")
                , new ArgsInfo("defendant"
                        , "defendant"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("plaintiff"
                        , "0"
                        , "company_send_announcement_del_0_plaintiff")
                , new ValueAlias("plaintiff"
                        , "1"
                        , "company_send_announcement_del_1_plaintiff")
                , new ValueAlias("defendant"
                        , "0"
                        , "company_send_announcement_del_0_defendant")
                , new ValueAlias("defendant"
                        , "1"
                        , "company_send_announcement_del_1_defendant")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("plaintiff_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("defendant_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_court_register", new SummaryArgs(
                "ng_rt_company_court_register"
                , Arrays.asList(new ArgsInfo("plaintiff"
                        , "plaintiff"
                        , "deleted")
                , new ArgsInfo("defendant"
                        , "defendant"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("plaintiff"
                        , "0"
                        , "company_court_register_del_0_plaintiff")
                , new ValueAlias("plaintiff"
                        , "1"
                        , "company_court_register_del_1_plaintiff")
                , new ValueAlias("defendant"
                        , "0"
                        , "company_court_register_del_0_defendant")
                , new ValueAlias("defendant"
                        , "1"
                        , "company_court_register_del_1_defendant")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("plaintiff_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("defendant_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("litigation_mediation", new SummaryArgs(
                "ng_rt_litigation_mediation"
                , Arrays.asList(new ArgsInfo("plaintiff"
                        , "plaintiff"
                        , "deleted")
                , new ArgsInfo("defendant"
                        , "defendant"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("plaintiff"
                        , "0"
                        , "litigation_mediation_del_0_plaintiff")
                , new ValueAlias("plaintiff"
                        , "1"
                        , "litigation_mediation_del_1_plaintiff")
                , new ValueAlias("defendant"
                        , "0"
                        , "litigation_mediation_del_0_defendant")
                , new ValueAlias("defendant"
                        , "1"
                        , "litigation_mediation_del_1_defendant")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("plaintiff_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("defendant_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("restrictions_on_exit", new SummaryArgs(
                "ng_rt_restrictions_on_exit"
                , Collections.singletonList(new ArgsInfo(
                "keyno"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "restrictions_on_exit_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "restrictions_on_exit_del_1"))
                , (j) -> Stream.of(getKey(j, "limited_person_pid"), getKey(j, "executed_person_keyno"))
                .distinct().filter(StringUtils::isNotBlank).collect(Collectors.toList())
        ));
        a.put("auction_tracking", new SummaryArgs(
                "ng_rt_auction_tracking"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "auction_tracking_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "auction_tracking_del_1"))
                , (j) -> parseJsonArray(j.getString("company_info"), "$.company_id")
        ));
        a.put("property_rights_transaction", new SummaryArgs(
                "ng_rt_property_rights_transaction"
                , Arrays.asList(new ArgsInfo(
                "subject"
                , "subject"
                , "deleted"), new ArgsInfo(
                "disposer"
                , "disposer"
                , "deleted"), new ArgsInfo(
                "dealer"
                , "dealer"
                , "deleted"))
                , Arrays.asList(new ValueAlias("subject"
                        , "0"
                        , "property_rights_transaction_del_0_subject")
                , new ValueAlias("subject"
                        , "1"
                        , "property_rights_transaction_del_1_subject")
                , new ValueAlias("disposer"
                        , "0"
                        , "property_rights_transaction_del_0_disposer")
                , new ValueAlias("disposer"
                        , "1"
                        , "property_rights_transaction_del_1_disposer")
                , new ValueAlias("dealer"
                        , "0"
                        , "property_rights_transaction_del_0_dealer")
                , new ValueAlias("dealer"
                        , "1"
                        , "property_rights_transaction_del_1_dealer")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("subject_name"), "$.keyno"));
            set.addAll(parseJsonArray(j.getString("disposer_info"), "$.keyno"));
            set.addAll(parseJsonArray(j.getString("dealer_info"), "$.keyno"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_equity_info", new SummaryArgs(
                "ng_rt_company_equity_info"
                , Arrays.asList(new ArgsInfo(
                "related"
                , "related"
                , "deleted"), new ArgsInfo(
                "pledgor"
                , "pledgor"
                , "deleted"), new ArgsInfo(
                "pledgee"
                , "pledgee"
                , "deleted"))
                , Arrays.asList(new ValueAlias("related"
                        , "0"
                        , "company_equity_info_del_0_related")
                , new ValueAlias("related"
                        , "1"
                        , "company_equity_info_del_1_related")
                , new ValueAlias("pledgor"
                        , "0"
                        , "company_equity_info_del_0_pledgor")
                , new ValueAlias("pledgor"
                        , "1"
                        , "company_equity_info_del_1_pledgor")
                , new ValueAlias("pledgee"
                        , "0"
                        , "company_equity_info_del_0_pledgee")
                , new ValueAlias("pledgee"
                        , "1"
                        , "company_equity_info_del_1_pledgee")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.add(j.getString("related_company_id"));
            set.addAll(parseJsonArray(j.getString("pledgor_info"), "$.pledgor_id"));
            set.addAll(parseJsonArray(j.getString("pledgee_info"), "$.pledgee_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_abnormal_info", new SummaryArgs(
                "ng_rt_company_abnormal_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_abnormal_info_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_abnormal_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_illegal_info", new SummaryArgs(
                "ng_rt_company_illegal_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_illegal_info_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_illegal_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_judicial_assistance", new SummaryArgs(
                "ng_rt_company_judicial_assistance"
                , Arrays.asList(new ArgsInfo("related"
                        , "related"
                        , "deleted")
                , new ArgsInfo("executed"
                        , "executed"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("related"
                        , "0"
                        , "company_judicial_assistance_del_0_related", true)
                , new ValueAlias("related"
                        , "1"
                        , "company_judicial_assistance_del_1_related")
                , new ValueAlias("executed"
                        , "0"
                        , "company_judicial_assistance_del_0_executed_person", true)
                , new ValueAlias("executed"
                        , "1"
                        , "company_judicial_assistance_del_1_executed_person")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(Collections.singletonList(getKey(j, "company_id")));
            set.addAll(Collections.singletonList(getKey(j, "executed_person_id")));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_brief_cancel_announcement", new SummaryArgs(
                "ng_rt_company_brief_cancel_announcement"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_brief_cancel_announcement_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_brief_cancel_announcement_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("high_tech_enterprise", new SummaryArgs(
                "ng_rt_high_tech_enterprise"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "high_tech_enterprise_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "high_tech_enterprise_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_liquidating_info", new SummaryArgs(
                "ng_rt_company_liquidating_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_liquidating_info_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_liquidating_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_license", new SummaryArgs(
                "ng_rt_company_license"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_license_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_license_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_check_info", new SummaryArgs(
                "ng_rt_company_check_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_check_info_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_check_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_punishment_info", new SummaryArgs(
                "ng_rt_company_punishment_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_punishment_info_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_punishment_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_double_random_check_info", new SummaryArgs(
                "ng_rt_company_double_random_check_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_double_random_check_info_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_double_random_check_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("cancellation_announcement", new SummaryArgs(
                "ng_rt_cancellation_announcement"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "cancellation_announcement_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "cancellation_announcement_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("online_retailers", new SummaryArgs(
                "ng_rt_online_retailers"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "online_retailers_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "online_retailers_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("finance_info_v3", new SummaryArgs(
                "ng_rt_finance_info_v3"
                , Arrays.asList(
                new ArgsInfo(
                        "company_id"
                        , ""
                        , "deleted"
                ),
                new ArgsInfo(
                        "company_id"
                        , ""
                        , "deleted"
                        , " and vendinc_dis = '1' "
                        , "dis"
                )
        )
                , Arrays.asList(new ValueAlias(
                        "del"
                        , "0"
                        , "finance_info_v3_del_0"),
                new ValueAlias(
                        "del"
                        , "1"
                        , "finance_info_v3_del_1"),
                new ValueAlias(
                        "dis"
                        , "0"
                        , "finance_info_v3_dis_0"),
                new ValueAlias(
                        "dis"
                        , "1"
                        , "finance_info_v3_dis_1")

        )
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("private_enterprise", new SummaryArgs(
                "ng_rt_private_enterprise"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "private_enterprise_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "private_enterprise_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_annual_report", new SummaryArgs(
                "ng_rt_company_annual_report"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_annual_report_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_annual_report_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("construction_qualification", new SummaryArgs(
                "ng_rt_construction_qualification"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "construction_qualification_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "construction_qualification_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("construction_person", new SummaryArgs(
                "ng_rt_construction_person"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "construction_person_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "construction_person_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("construction_project", new SummaryArgs(
                "ng_rt_construction_project"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "construction_project_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "construction_project_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));

        a.put("company_bid_new", new SummaryArgs(
                "ng_rt_company_bid_new"
                , Arrays.asList(new ArgsInfo("supplier"
                        , "supplier"
                        , "deleted")
                , new ArgsInfo("purchaser"
                        , "purchaser"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("supplier"
                        , "0"
                        , "company_bid_new_del_0_supplier")
                , new ValueAlias("supplier"
                        , "1"
                        , "company_bid_new_del_1_supplier")
                , new ValueAlias("purchaser"
                        , "0"
                        , "company_bid_new_del_0_purchaser")
                , new ValueAlias("purchaser"
                        , "1"
                        , "company_bid_new_del_1_purchaser")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("supplier_info"), "$.keyno"));
            set.addAll(parseJsonArray(j.getString("purchaser_info"), "$.keyno"));
            return new ArrayList<>(set);
        }
        ));
        a.put("reduction_registered_capital_info", new SummaryArgs(
                "ng_rt_reduction_registered_capital_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "reduction_registered_capital_info_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "reduction_registered_capital_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));

        a.put("increase_registered_capital_info", new SummaryArgs(
                "ng_rt_increase_registered_capital_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "increase_registered_capital_info_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "increase_registered_capital_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("general_taxpayer", new SummaryArgs(
                "ng_rt_general_taxpayer"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "general_taxpayer_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "general_taxpayer_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_mortgage_info", new SummaryArgs(
                "company_mortgage_info"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_mortgage_info_del_0", true),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_mortgage_info_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("wenshu_detail_v2", new SummaryArgs(
                "ng_rt_wenshu_detail_v2"
                , Arrays.asList(new ArgsInfo("plaintiff"
                        , "plaintiff"
                        , "deleted")
                , new ArgsInfo("defendant"
                        , "defendant"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("plaintiff"
                        , "0"
                        , "wenshu_detail_v2_del_0_plaintiff", true)
                , new ValueAlias("plaintiff"
                        , "1"
                        , "wenshu_detail_v2_del_1_plaintiff")
                , new ValueAlias("defendant"
                        , "0"
                        , "wenshu_detail_v2_del_0_defendant", true)
                , new ValueAlias("defendant"
                        , "1"
                        , "wenshu_detail_v2_del_1_defendant")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.addAll(parseJsonArray(j.getString("plaintiff_info"), "$.litigant_id"));
            set.addAll(parseJsonArray(j.getString("defendant_info"), "$.litigant_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_env_punishment", new SummaryArgs(
                "ng_rt_company_env_punishment"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_env_punishment_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_env_punishment_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_own_tax", new SummaryArgs(
                "ng_rt_company_own_tax"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_own_tax_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_own_tax_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_land_announcement", new SummaryArgs(
                "ng_rt_company_land_announcement"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_land_announcement_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_land_announcement_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_land_publicity", new SummaryArgs(
                "ng_rt_company_land_publicity"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_land_publicity_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_land_publicity_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_land_transfer", new SummaryArgs(
                "ng_rt_company_land_transfer"
                , Arrays.asList(new ArgsInfo("user_pre"
                        , "user_pre"
                        , "deleted")
                , new ArgsInfo("user_now"
                        , "user_now"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("user_pre"
                        , "0"
                        , "company_land_transfer_del_0_user_pre")
                , new ValueAlias("user_pre"
                        , "1"
                        , "company_land_transfer_del_1_user_pre")
                , new ValueAlias("user_now"
                        , "0"
                        , "company_land_transfer_del_0_user_now")
                , new ValueAlias("user_now"
                        , "1"
                        , "company_land_transfer_del_1_user_now")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.add(j.getString("pre_keyno"));
            set.add(j.getString("now_keyno"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_equity_pledge_holder", new SummaryArgs(
                "ng_rt_company_equity_pledge_holder"
                , Arrays.asList(new ArgsInfo("company"
                        , "company"
                        , "deleted")
                , new ArgsInfo("holder"
                        , "holder"
                        , "deleted")
                , new ArgsInfo("pledgee"
                        , "pledgee"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("company"
                        , "0"
                        , "company_equity_pledge_holder_del_0_company")
                , new ValueAlias("company"
                        , "1"
                        , "company_equity_pledge_holder_del_1_company")
                , new ValueAlias("holder"
                        , "0"
                        , "company_equity_pledge_holder_del_0_holder")
                , new ValueAlias("holder"
                        , "1"
                        , "company_equity_pledge_holder_del_1_holder")
                , new ValueAlias("pledgee"
                        , "0"
                        , "company_equity_pledge_holder_del_0_pledgee")
                , new ValueAlias("pledgee"
                        , "1"
                        , "company_equity_pledge_holder_del_1_pledgee")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.add(j.getString("company_id"));
            set.add(j.getString("holder_id"));
            set.add(j.getString("pledgee_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_land_mortgage", new SummaryArgs(
                "ng_rt_company_land_mortgage"
                , Arrays.asList(new ArgsInfo("mortgagor"
                        , "mortgagor"
                        , "deleted")
                , new ArgsInfo("mortgagee"
                        , "mortgagee"
                        , "deleted")
        )
                , Arrays.asList(new ValueAlias("mortgagor"
                        , "0"
                        , "company_land_mortgage_del_0_mortgagor")
                , new ValueAlias("mortgagor"
                        , "1"
                        , "company_land_mortgage_del_1_mortgagor")
                , new ValueAlias("mortgagee"
                        , "0"
                        , "company_land_mortgage_del_0_mortgagee")
                , new ValueAlias("mortgagee"
                        , "1"
                        , "company_land_mortgage_del_1_mortgagee")
        )
                , (j) -> {
            Set<String> set = new HashSet<>();
            set.add(j.getString("mortgagor_company_id"));
            set.add(j.getString("mortgagee_company_id"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_equity_pledge", new SummaryArgs(
                "ng_rt_company_equity_pledge"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_equity_pledge_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_equity_pledge_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_tax_contravention", new SummaryArgs(
                "ng_rt_company_tax_contravention"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_tax_contravention_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_tax_contravention_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_punishment_info_creditchina", new SummaryArgs(
                "ng_rt_company_punishment_info_creditchina"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_punishment_info_creditchina_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_punishment_info_creditchina_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_license_creditchina", new SummaryArgs(
                "ng_rt_company_license_creditchina"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_license_creditchina_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_license_creditchina_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_license_entpub", new SummaryArgs(
                "ng_rt_company_license_entpub"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_license_entpub_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_license_entpub_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_public_announcement", new SummaryArgs(
                "ng_rt_company_public_announcement"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_public_announcement_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_public_announcement_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_tax", new SummaryArgs(
                "ng_rt_company_tax"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_tax_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_tax_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        a.put("company_patent", new SummaryArgs(
                "ng_rt_company_patent"
                , Arrays.asList(new ArgsInfo("applicant_name"
                , ""
                , "deleted")
        )
                , Arrays.asList(new ValueAlias(""
                        , "0"
                        , "company_patent_del_0")
                , new ValueAlias(""
                        , "1"
                        , "company_patent_del_1")
        )
                , (j) -> {
            Set<String> set = new HashSet<>(parseJsonArray(j.getString("applicant_name_info"), "$.keyno"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_copyright_reg", new SummaryArgs(
                "ng_rt_company_copyright_reg"
                , Arrays.asList(new ArgsInfo("author_nationality"
                , ""
                , "deleted")
        )
                , Arrays.asList(new ValueAlias(""
                        , "0"
                        , "company_copyright_reg_del_0")
                , new ValueAlias(""
                        , "1"
                        , "company_copyright_reg_del_1")
        )
                , (j) -> {
            Set<String> set = new HashSet<>(parseJsonArray(j.getString("author_nationality_info"), "$.keyno"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_copyright_works", new SummaryArgs(
                "ng_rt_company_copyright_works"
                , Arrays.asList(new ArgsInfo("author"
                , ""
                , "deleted")
        )
                , Arrays.asList(new ValueAlias(""
                        , "0"
                        , "company_copyright_works_del_0")
                , new ValueAlias(""
                        , "1"
                        , "company_copyright_works_del_1")
        )
                , (j) -> {
            Set<String> set = new HashSet<>(parseJsonArray(j.getString("author_info"), "$.keyno"));
            return new ArrayList<>(set);
        }
        ));
        a.put("company_profile", new SummaryArgs(
                "ng_rt_company_profile"
                , Collections.singletonList(new ArgsInfo(
                "company_id"
                , ""
                , "deleted"))
                , Arrays.asList(new ValueAlias(
                        ""
                        , "0"
                        , "company_profile_del_0"),
                new ValueAlias(
                        ""
                        , "1"
                        , "company_profile_del_1"))
                , (j) -> Collections.singletonList(j.getString("company_id"))
        ));
        return a;
    }

    @Data
    @NoArgsConstructor
    public static class ArgsInfo {
        public String filterField;
        private String category;
        private String groupField;
        private String condition = "";
        private String type = "del";


        public ArgsInfo(String filterField, String category, String groupField) {
            this.filterField = filterField;
            this.category = category;
            this.groupField = groupField;
        }

        public ArgsInfo(String filterField, String category, String groupField, String condition, String type) {
            this.filterField = filterField;
            this.category = category;
            this.groupField = groupField;
            this.condition = condition;
            this.type = type;
        }
    }

    @Data
    @NoArgsConstructor
    public static class ValueAlias {
        private String category;
        private String value;
        private String alias;
        private boolean isSum = false;

        public ValueAlias(String category, String value, String alias) {
            this.category = category;
            this.value = value;
            this.alias = alias;
        }

        public ValueAlias(String category, String value, String alias, Boolean isSum) {
            this.category = category;
            this.value = value;
            this.alias = alias;
            this.isSum = isSum;
        }
    }
}
