package com.winhc.task.configuration;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.nio.client.HttpAsyncClientBuilder;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import java.util.stream.Stream;

/**
 * @Author: XuJiakai
 * @Date: 2020/9/15 20:01
 * @Description:
 */
@Configuration
public class ElasticSearchConfiguration {
    @Value("${es.username}")
    private String username;
    @Value("${es.password}")
    private String password;
    @Value("${es.host}")
    private String host;

    @Value("${es.username1}")
    private String username1;
    @Value("${es.password1}")
    private String password1;
    @Value("${es.host1}")
    private String host1;

    @Value("${es.schema:http}")
    String schema;
    @Value(value = "${es.connect-timeout:100000}")
    String connectTimeout;
    @Value(value = "${es.socket-timeout:600000}")
    String socketTimeout;
    @Value(value = "${es.connection-request-timeout:50000}")
    String connectionRequestTimeout;
    @Value(value = "${es.max-conn-total:100}")
    String maxConnTotal;
    @Value(value = "${es.max-conn-per-route:100}")
    String maxConnPerRoute;

    @Bean
    public RestClient bean() {
        final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
        credentialsProvider.setCredentials(AuthScope.ANY,
                new UsernamePasswordCredentials(username, password));
        // 单击所创建的Elasticsearch实例ID，在基本信息页面获取公网地址，即为HOST。
        return RestClient.builder(new HttpHost(host, 9200))
                .setHttpClientConfigCallback(new RestClientBuilder.HttpClientConfigCallback() {
                    @Override
                    public HttpAsyncClientBuilder customizeHttpClient(HttpAsyncClientBuilder httpClientBuilder) {
                        return httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider);
                    }
                }).build();
    }


    @Bean("v6")
    @Primary
    public RestHighLevelClient getClient() {
        HttpHost[] httpHosts = Stream.of(host.split(",")).map(host -> {
            String[] split = host.split(":");
            return new HttpHost(split[0], 9200, schema);
        }).toArray(HttpHost[]::new);

        // 阿里云Elasticsearch集群需要basic auth验证。
        final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
        //访问用户名和密码为您创建阿里云Elasticsearch实例时设置的用户名和密码，也是Kibana控制台的登录用户名和密码。
        credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(username, password));


        return new RestHighLevelClient(RestClient
                .builder(httpHosts)
                .setMaxRetryTimeoutMillis(60000 * 3)
                .setRequestConfigCallback(builder -> {
                    builder.setConnectTimeout(1000 * 3);
                    builder.setSocketTimeout(60000 * 3);
                    builder.setConnectionRequestTimeout(0);
                    return builder;
                })
                .setHttpClientConfigCallback(httpAsyncClientBuilder -> {
                    httpAsyncClientBuilder.setDefaultCredentialsProvider(credentialsProvider);
                    httpAsyncClientBuilder.setMaxConnTotal(100 * 3);
                    httpAsyncClientBuilder.setMaxConnPerRoute(100 * 3);
                    return httpAsyncClientBuilder;
                }).build()
        );
    }

    @Bean("v5")
    public RestHighLevelClient getOldClient() {
        HttpHost[] httpHosts = Stream.of(host1.split(",")).map(host -> {
            String[] split = host.split(":");
            return new HttpHost(split[0], 9200, schema);
        }).toArray(HttpHost[]::new);

        // 阿里云Elasticsearch集群需要basic auth验证。
        final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
        //访问用户名和密码为您创建阿里云Elasticsearch实例时设置的用户名和密码，也是Kibana控制台的登录用户名和密码。
        credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(username1, password1));


        return new RestHighLevelClient(RestClient
                .builder(httpHosts)
                .setMaxRetryTimeoutMillis(60000 * 3)
                .setRequestConfigCallback(builder -> {
                    builder.setConnectTimeout(1000 * 3);
                    builder.setSocketTimeout(60000 * 3);
                    builder.setConnectionRequestTimeout(0);
                    return builder;
                })
                .setHttpClientConfigCallback(httpAsyncClientBuilder -> {
                    httpAsyncClientBuilder.setDefaultCredentialsProvider(credentialsProvider);
                    httpAsyncClientBuilder.setMaxConnTotal(100 * 3);
                    httpAsyncClientBuilder.setMaxConnPerRoute(100 * 3);
                    return httpAsyncClientBuilder;
                }).build()
        );
    }
}
