package com.winhc.task.dao.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.winhc.task.dao.SearchDao;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.rescore.RescoreBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;

/**
 * @author: XuJiakai
 * 2020/11/20 15:16
 */
@Slf4j
@Repository
@AllArgsConstructor
public class SearchDaoImpl implements SearchDao {
    private RestHighLevelClient restHighLevelClient;

//    @Qualifier("v6")
//    @Autowired
//    RestHighLevelClient getClient;


    private ObjectMapper mapper;
    private static final TypeReference<HashMap<String, Object>> typeRef
            = new TypeReference<HashMap<String, Object>>() {
    };

    @SneakyThrows
    @Override
    public Object search(String index, String type, QueryBuilder query, SortBuilder sortBuilder, FetchSourceContext fetchSourceContext, int from, int size) {
        return search(index, type, query, null, sortBuilder, fetchSourceContext, from, size);
    }

    @Override
    public Object search(String index, String type, QueryBuilder query, int from, int size) {
        return search(index, type, query, null, null, from, size);
    }

    @SneakyThrows
    @Override
    public Object search(String index, String type, QueryBuilder query, RescoreBuilder rescoreBuilder, SortBuilder sortBuilder, FetchSourceContext fetchSourceContext, int from, int size) {
      /*  HighlightBuilder.Field query1 = new HighlightBuilder.Field("history_name.value").highlightQuery(QueryBuilders.matchQuery("history_name.value", "华为"));
        HighlightBuilder.Field query3 = new HighlightBuilder.Field("history_name.value.keyword").highlightQuery(QueryBuilders.matchQuery("history_name.value", "华为").boost(10000));
        HighlightBuilder.Field query2 = new HighlightBuilder
                .Field("cname.value")
                .highlightQuery(QueryBuilders.matchQuery("cname.value", "华为").boost(1000));*/
        HighlightBuilder highlightBuilder = new HighlightBuilder()
//                .field(query1)
//                .field(query2)
//                .field(query3)
                .preTags("<font color='red'>")
                .postTags("</font>")
                .order("score");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(query)
                .from(from)
                .size(size)
                .highlighter(highlightBuilder);

        if (rescoreBuilder != null) {
            searchSourceBuilder.addRescorer(rescoreBuilder)
//                    .addRescorer(new QueryRescorerBuilder(query))
            ;
        }
        if (fetchSourceContext != null) {
            searchSourceBuilder.fetchSource(fetchSourceContext);
        }
        if (sortBuilder != null) {
            searchSourceBuilder.sort(sortBuilder)
//                    .trackScores(true)
            ;
        }

        SearchRequest searchRequest = new SearchRequest()
                .indices(index)
                .types(type)
                .source(searchSourceBuilder);

        SearchResponse search = restHighLevelClient.search(searchRequest);
        return mapper.readValue(search.toString(), typeRef);
    }

    @SneakyThrows
    @Override
    public boolean deleteByIds(String index, String type, List<String> ids) {
        BulkRequest bulkRequest = new BulkRequest();
        for (String id : ids) {
            DeleteRequest deleteRequest = new DeleteRequest(index, type, id);
            bulkRequest.add(deleteRequest);
        }
        restHighLevelClient.bulk(bulkRequest);
        return true;
    }


    private final RestClient restClient;

    @SneakyThrows
    public Object test() {
        Response get = restClient.performRequest("get", "_cat/indices/judicial_case_v*?h=index");
        String s = EntityUtils.toString(get.getEntity());
        System.out.println(s);

        return null;
    }
}
