package com.winhc.task.job;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.util.CharsetUtil;
import com.alibaba.fastjson.JSONObject;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.InsertManyOptions;
import com.mongodb.client.result.InsertManyResult;
import com.winhc.task.common.ArgsCompanyJob;
import com.winhc.task.common.Constant;
import com.winhc.task.framework.es.EsFastScan;
import com.winhc.task.util.QueryBuilderUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.bson.BsonValue;
import org.bson.Document;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.WrapperQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;


@Slf4j
@Component
@AllArgsConstructor
public class EsScanSummaryJob {
    private RestHighLevelClient restHighLevelClient;
    private final MongoTemplate mongoTemplate;

//    @Autowired
//    @Qualifier(value = "v6")
//    RestHighLevelClient getClient;

    public void start() throws IOException {
        String dsl0 = "{\n" +
                "  \"query\": {\n" +
                "    \"terms\": {\n" +
                "      \"cname.show.keyword\": [\n" +
                "        \"沈阳国际软件园产业发展有限公司\",\n" +
                "        \"三橡股份有限公司\",\n" +
                "        \"沈阳飞行船数码喷印设备有限公司\",\n" +
                "        \"沈阳中之杰流体控制系统有限公司\",\n" +
                "        \"沈阳中北通磁科技股份有限公司\",\n" +
                "        \"沈阳沈大内窥镜有限公司\",\n" +
                "        \"沈阳化工集团有限公司\",\n" +
                "        \"华晨宝马汽车有限公司\",\n" +
                "        \"特变电工沈阳变压器集团有限公司\",\n" +
                "        \"沈阳电缆厂销售处\"\n" +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";
        String dsl1 = "{\n" +
                "  \"query\": {\n" +
                "    \"terms\": {\n" +
                "      \"_id\": [\n" +
                "        &ids " +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";
        scan(dsl0, dsl1);
    }

    public void scan(String dsl0, String dsl1) throws IOException {

        SearchRequest searchRequest = new SearchRequest("winhc_index_rt_company").types("company");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.size(10000);
        if (StringUtils.isNotEmpty(dsl0)) {
            WrapperQueryBuilder query = QueryBuilderUtils.getQuery(JSONObject.parseObject(dsl0));
            searchSourceBuilder.query(query);
        }
        searchRequest.source(searchSourceBuilder);

        SearchResponse searchResponse = restHighLevelClient.search(searchRequest);

        List<String> ids = Arrays.stream(searchResponse.getHits().getHits()).map(SearchHit::getId).distinct().collect(Collectors.toList());
        System.out.println(ids);
        //补充维度缺失
        List<String> id_add = Arrays.asList(
                "91cf436e1959f7d8ccf3baf275f8ba17",
                "575c80b44ab6a8d9ef8bcc101241c074",
                "1aefccaa52fb392de544c641c7977e2b",
                "3b316c0d15f765450f315d217f2dcaa5",
                "baaf16ec0963bcecf62286fa1a38d5f5",
                "b70ee50a247536457489c20c8a45d8c6",
                "b5c2e4002f2caea6c2200b2d61c496f9",
                "210a3c670ed4036ee8d89d0e33de52c4",
                "4a80cef767b7979ec0f19acb2e5f1edb",
                "843eb83e1b238c6a34cbf02c44d31b32",
                "190f4302a77d2c5fec225d19c173f76e",
                "dcd5b075ccef4385883ef8e8688a44c1",
                "57453fd00b48456e33bef2105058a499",
                "be9b3fb64b79d27eec53c282bdc86c11",
                "6820fae3c815fb899214617b54efadf7",
                "2cc25a8e43426abc893429b6b6f9cd22");//询价机构

        //v9子表维度
        List<String> id_add2 = Arrays.asList(
                "f638994f9b45aed70b18f057d5215b97",
                "b41f46dad10f3e311aeb5101fb5e1a99",
                "98c5a5bb0405681672a8f1b06c30eaf2",
                "c40d568887c2c24c1446c747847de3ab",
                "79dba12967e83e080c3493ad4826e5dd",
                "8ac84008ff8e0682fd5b9d385e9abcf1",
                "bda6c23ecfeceff5d867a7fe922068bb",
                "63ff6b100d7e1729fb1e18123ed88831",
                "7649322a219335dfdfc3eea26c1d7dcf",
                "285e6cbb96c57776c80b067c25aca51c",
                "f5f04a699e0f488e46f676f0c99e14e6",
                "9bf46052cab0fd62a81827ebe63a7be2",
                "54038eefa74235b6031653fac59cd50f",
                "609f846039bf87d3af3797e78476b372",
                "af01810f76ba395a798cb9dafdebfeb2");

        //v8 维度
        List<String> id_add3 = Arrays.asList(
                "0d21340a46b1a749e17b8fb260e1b6b6",
                "ca19f7ef107045af8d66e4302261204f",
                "aa0b8980af16ebbccfa39f2ca97891cc",
                "653acbb94796852c1382f49ee445d25e",
                "64eac2f662d5826c475a0cce1120e85d",
                "c76b33a126a0a8a27cdc428b5395b1ec",
                "dae0b5bea9342644e8acdbcca3f69a60",
                "8e2298078c52221f5837909933732f70",
                "5cb3e29512491ab8f2b16df5aa9c1d0e");
        //v8子表维度
        List<String> id_add4 = Arrays.asList(
                "c1979818791446c47ae15885564bfcef", "0088f0bbc061e2d57ad301822ab92a44",
                "b14b2e11bd959af3a9f022e8c7eb3ec5", "00329ba5d9ff6c7549496a9291791801",
                "5a286c96967bc3516e8c8f055df53da4", "05ec800bff8f224b85329a86817149a4",
                "73a1a35da3c1165eaa3ae385854405cb", "0000c8416a8f9b8e51b9d0bf8a7a5b95",
                "dc4273e21f9d7b520ce01cb80b02f626", "03272efaf00d239fa240b03ee19a7ea7"
        );

        ids.addAll(id_add);
        ids.addAll(id_add2);
        ids.addAll(id_add3);
        ids.addAll(id_add4);
//        全部企业id
        String all_ids = ids.stream().distinct().collect(Collectors.joining("\n"));
        System.out.println(ids.stream().distinct().collect(Collectors.joining("\n")));
        //path指定路径下的文件如不存在，则创建
        try {
            FileUtil.writeString(all_ids, Constant.all_ids_path, CharsetUtil.UTF_8);
        } catch (IORuntimeException e) {
            //抛出一个运行时异常(直接停止掉程序)
            throw new RuntimeException("运行时异常", e);
        }

        String r_ids = ids.stream().distinct().collect(Collectors.joining("\",\"", "\"", "\""));
        String dsl = dsl1.replace("&ids", r_ids);

        Consumer<SearchHit[]> func = list -> {
            List<String> li = Arrays.stream(list).map(d -> {
                String index = d.getIndex();
                return index.substring(0, index.lastIndexOf("_")).replace("ng_rt_summary_", "");
            }).distinct().collect(Collectors.toList());

            List<String> tns = ArgsCompanyJob.JOB_ARGS.keySet()
                    .stream()
                    .filter(Constant.tns_out::contains)
                    .distinct()
                    .collect(Collectors.toList());
            String re = tns.stream().filter(m -> !li.contains(m)).collect(Collectors.joining("\n"));
            System.out.println(re);
        };
        new EsFastScan(restHighLevelClient, func, "ng_rt_summary_company", "_doc", dsl, null).scan();
    }

}
