package com.winhc.task.service.impl;

import com.winhc.task.service.InnerSearchCompanyService;
import com.winhc.task.util.CompanyNameUtils;
import com.winhc.task.dao.SearchDao;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MultiMatchQueryBuilder;
import org.springframework.stereotype.Service;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author: XuJiakai
 * 2021/4/23 19:01
 */
@Slf4j
@Service
@AllArgsConstructor
public class InnerSearchCompanyServiceImpl implements InnerSearchCompanyService {
    private final SearchDao searchDao;

    public static final String index = "winhc-company-v8";
    public static final String type = "company";


    @Override
    public Object searchCompanyName(String companyName) {
        return searchDao.search(index, type, getBoolQuery(companyName), 0, 5);
    }

    private BoolQueryBuilder getBoolQuery(String companyName) {
        BoolQueryBuilder boolQuery = boolQuery();
        String content = CompanyNameUtils.cleanup(companyName);


        boolQuery.should(termQuery("cname.value.keyword", content).boost(1000));
        boolQuery.should(termQuery("history_name.value.keyword", content).boost(1000));
        boolQuery.should(multiMatchQuery(companyName)
                .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                .minimumShouldMatch("5<85%")
                .tieBreaker(0.3F)
                .field("cname.value", 16)
                .field("history_name.value", 12)
                .field("cname.value.pinyin", 6)
                .field("history_name.value.pinyin", 6)

        );

        return boolQuery;
    }
}
