package com.winhc.task.service.impl;

import com.winhc.task.service.SearchService;
import com.winhc.task.dao.impl.SearchDaoImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.common.lucene.search.function.CombineFunction;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MultiMatchQueryBuilder;
import org.elasticsearch.index.query.Operator;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.functionscore.FunctionScoreQueryBuilder;
import org.elasticsearch.index.query.functionscore.ScriptScoreFunctionBuilder;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.sort.ScriptSortBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.stereotype.Service;
import java.util.HashMap;
import java.util.Map;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author: XuJiakai
 * 2020/12/7 10:58
 */
@Slf4j
@Service(SearchV7ServiceImpl.index)
@AllArgsConstructor
public class SearchV7ServiceImpl implements SearchService {


    private SearchDaoImpl searchDao;
    private static final ScriptSortBuilder scriptSortBuilder = fastDefaultSort();

    public static final String index = "winhc-company-v7";
    public static final String type = "company";

    private static final String[] includes = new String[]{"cname", "estiblish_time", "reg_status", "company_type", "province_code", "reg_capital", "logo"};
    private static final FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, null);


    private static final String[] includes_tips = new String[]{"cname.show"};
    private static final FetchSourceContext fetchSourceContext_tips = new FetchSourceContext(true, includes_tips, null);

    private BoolQueryBuilder getBoolQuery(String content) {
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
        MultiMatchQueryBuilder multiMatchQueryBuilder = multiMatchQuery(content)
                .operator(Operator.AND)
                .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                .minimumShouldMatch("5<80%")
                .tieBreaker(0.3F);

        boolQuery.should(termQuery("cname.value.keyword", content).boost(100));
        boolQuery.should(termQuery("history_name.value.keyword", content).boost(100));

        multiMatchQueryBuilder
                .field("cname.value", 16)
                .field("history_name.value", 12)
        ;


        boolQuery.should(multiMatchQueryBuilder);
        BoolQueryBuilder boolQuery2 = QueryBuilders.boolQuery();
        boolQuery2.must(boolQuery);
        boolQuery2.mustNot(QueryBuilders.existsQuery("current_id"));

        return boolQuery2;

    }


    private static ScriptSortBuilder fastDefaultSort() {
        Map<String, Object> params = new HashMap<>();
        String script_inline =
                "if(doc['cname.value.keyword'].value==null||doc['cname.value.keyword'].value.length()==3){" +
                        "return 0.01;" +
                        "}" +
                        "if(doc['cname.value.keyword'].value.length()<=3){" +
                        "return 0.3;" +
                        "}" +
                        "if(doc['reg_status'].value==null || doc['reg_status'].value.contains('销')){" +
                        "return 1;" +
                        "}" +
                        "double a = doc['reg_capital_amount']==null?0.0:doc['reg_capital_amount'].value>1000000000000.0?1000000000000.0:doc['reg_capital_amount'].value;" +
                        "double w = Math.log(a/10000000+1)+1;" +
                        "if(doc['company_type'].value=='1'){" +
                        "w=w+3;" +
                        "}" +
                        "return w;";

        Script script = new Script(ScriptType.INLINE, "painless", script_inline, params);
        return new ScriptSortBuilder(script, ScriptSortBuilder.ScriptSortType.NUMBER).order(SortOrder.DESC);

    }

    @Override
    public Object tips(String s) {
        BoolQueryBuilder boolQuery = getBoolQuery(s);
        FunctionScoreQueryBuilder function = functionScoreQuery(boolQuery, new ScriptScoreFunctionBuilder(scriptSortBuilder.script())
        )
                .boostMode(CombineFunction.MULTIPLY);

        return searchDao.search(index, type, function, null, fetchSourceContext_tips, 0, 5);
    }

    @Override
    public Object controlGroup(String s) {
        return searchDao.search(index, type, getBoolQuery(s), null, fetchSourceContext, 0, 10);
    }

    @Override
    public Object query(String s, int from, int size) {
        BoolQueryBuilder boolQuery = getBoolQuery(s);
        FunctionScoreQueryBuilder function = functionScoreQuery(boolQuery, new ScriptScoreFunctionBuilder(scriptSortBuilder.script())
        )
                .boostMode(CombineFunction.MULTIPLY);

        return searchDao.search(index, type, function, null, fetchSourceContext, from, size);
    }
}
