package com.winhc.task.service.impl;

import com.winhc.task.service.SearchService;
import com.winhc.task.util.SortUtil;
import com.winhc.task.dao.SearchDao;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.common.lucene.search.function.CombineFunction;
import org.elasticsearch.common.lucene.search.function.FiltersFunctionScoreQuery;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MultiMatchQueryBuilder;
import org.elasticsearch.index.query.Operator;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.functionscore.FunctionScoreQueryBuilder;
import org.elasticsearch.index.query.functionscore.ScriptScoreFunctionBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.sort.ScriptSortBuilder;
import org.springframework.stereotype.Service;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * @author: XuJiakai
 * 2020/12/8 19:12
 */
@Slf4j
@Service(SearchV8SimpServiceImpl.index)
@AllArgsConstructor
public class SearchV8SimpServiceImpl implements SearchService {
    private SearchDao searchDao;

    public static final String index = "winhc-company-v8-simp";
    public static final String type = "company";

    private static final String[] includes_tips = new String[]{"cname.show", "new_cid"};
    private static final FetchSourceContext fetchSourceContext_tips = new FetchSourceContext(true, includes_tips, null);


    @Override
    public Object tips(String s) {
        BoolQueryBuilder boolQuery = getBoolQuery(s);
        ScriptSortBuilder scriptSortBuilder = SortUtil.getInstance().fastSort;
        FunctionScoreQueryBuilder function = functionScoreQuery(boolQuery, new ScriptScoreFunctionBuilder(scriptSortBuilder.script())
        )
                .scoreMode(FiltersFunctionScoreQuery.ScoreMode.SUM)
                .boostMode(CombineFunction.MULTIPLY);

        Object search = searchDao.search(index, type, function, null, fetchSourceContext_tips, 0, 5);
        return search;
    }

    @Override
    public Object controlGroup(String s) {
        return searchDao.search(index, type, getBoolQuery(s), null, fetchSourceContext_tips, 0, 10);
    }

    @Override
    public Object query(String s, int from, int size) {
        BoolQueryBuilder boolQuery = getBoolQuery(s);
        ScriptSortBuilder scriptSortBuilder = SortUtil.getInstance().fastSort;
        FunctionScoreQueryBuilder function = functionScoreQuery(boolQuery, new ScriptScoreFunctionBuilder(scriptSortBuilder.script())
        )
                .boostMode(CombineFunction.MULTIPLY);

        Object search = searchDao.search(index, type, function, null, fetchSourceContext_tips, from, size);
        return search;
    }


    private BoolQueryBuilder getBoolQuery(String content) {
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();

        // 公司现用名*100
        boolQuery.should(termQuery("cname.value.keyword", content).boost(100));
        // 公司曾用名*100
        boolQuery.should(termQuery("history_name.value.keyword", content).boost(100));


        // sum(商标全匹配*20，产品全匹配*40)
        boolQuery.should(
                boolQuery()
                        .should(termQuery("app_info.keyword", content).boost(40))
                        .should(termQuery("company_tm.keyword", content).boost(20))

        );

        // sum(商标*5，产品信息*15，商标全匹配*20，产品全匹配*40)
        boolQuery.should(
                boolQuery()

                        .should(matchQuery("app_info", content).boost(15))
                        .should(matchQuery("company_tm", content).boost(5))

                        .should(termQuery("app_info.keyword", content).boost(40))
                        .should(termQuery("company_tm.keyword", content).boost(20))
                        // 加入最小匹配度
                        .minimumShouldMatch("5<80%")
        );

        // 0.7 * max（公司现用名*16,曾用名*12）+ 0.3 * sum(公司现用名*16,曾用名*12)
        MultiMatchQueryBuilder multiMatchQueryBuilder = multiMatchQuery(content)
                .operator(Operator.AND)
                .type(MultiMatchQueryBuilder.Type.CROSS_FIELDS)
                .minimumShouldMatch("5<80%")
                .tieBreaker(0.3F)
                .field("cname.value", 16)
                .field("history_name.value", 12);

        boolQuery.should(multiMatchQueryBuilder);

        BoolQueryBuilder boolQuery2 = boolQuery()
                // 过deleted非0数据
                .filter(termQuery("deleted", "0"))
                .must(boolQuery);
        return boolQuery2;

    }

}
