package com.winhc.task.util;

import com.alibaba.hologres.client.HoloClient;
import com.alibaba.hologres.client.HoloConfig;
import com.alibaba.hologres.client.exception.HoloClientException;
import com.alibaba.hologres.client.model.WriteMode;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.stream.Collectors;

/**
 * @Author: π
 */
@Slf4j
public class HoloUtils {
    private static final String url;

    static {
        if (BaseUtils.isWindows()) {
            url = "jdbc:postgresql://hgprecn-cn-zvp2dvgsr005-cn-shanghai.hologres.aliyuncs.com:80/winhc_ng_rt";
        } else {
            url = "jdbc:postgresql://hgprecn-cn-zvp2dvgsr005-cn-shanghai-vpc.hologres.aliyuncs.com:80/winhc_ng_rt";
        }
    }

    private static final String username = "LTAI4G4n7pAW8tUbJVkkZQPD";
    private static final String password = "uNJOBskzcDqHq1TYG3m2rebR4c1009";

    public static HoloClient init() throws HoloClientException {
        HoloConfig config = new HoloConfig();
        config.setJdbcUrl(url);
        config.setUsername(username);
        config.setPassword(password);
        config.setFlushMaxWaitMs(60000 * 10L);
        config.setScanTimeoutSeconds(60 * 10);
        config.setBinlogReadTimeoutSeconds(60 * 10);
        config.setConnectionMaxIdleMs(60000 * 10L);
        config.setMetaCacheTTL(60000 * 10L);
        config.setWriteMode(WriteMode.INSERT_OR_REPLACE);//配置主键冲突时策略
        return new HoloClient(config);
    }

    @SneakyThrows
    public static void exexSql(HoloClient client, String pre_sql) {
        log.info("start exec holo sql : \n{}", pre_sql);
        client.sql(conn -> {
            Statement stat = conn.createStatement();
            return stat.execute(pre_sql);
        })
                .get();
                //.get(1000 * 60 * 30, TimeUnit.MILLISECONDS);
    }

    @SneakyThrows
    public static void exexSql2(HoloClient client, String pre_sql) {
        log.info("start exec holo sql : \n{}", pre_sql);
        client.sql(conn -> {
            Statement stat = conn.createStatement();
            return stat.execute(pre_sql);
        }).get(1000 * 60 * 30, TimeUnit.MILLISECONDS);
    }

    @SneakyThrows
    public static <T> List<T> exec(HoloClient holoClient, String sql, Class<T> typeClazz) {
        List<T> list = new ArrayList<>();
        Field[] declaredFields = typeClazz.getDeclaredFields();
        Method[] declaredMethods = typeClazz.getDeclaredMethods();
        Set<String> methodSet = Arrays.stream(declaredMethods).map(Method::getName).collect(Collectors.toSet());
        try {
            ResultSet resultSet1 = holoClient.sql(conn -> {
                ResultSet resultSet;
                try (Statement stat = conn.createStatement()) {
                    resultSet = stat.executeQuery(sql);
                }
                return resultSet;
            }).get();
            Set<String> fieldSet = getFieldSet(resultSet1, typeClazz);

            while (resultSet1.next()) {
                T instance = typeClazz.newInstance();
                for (Field field : declaredFields) {
                    Class<?> type = field.getType();
                    String fieldSetName = "set" + StringUtils.capitalize(field.getName());
                    if (!methodSet.contains(fieldSetName) || !fieldSet.contains(field.getName())) {
                        continue;
                    }
                    Method fieldSetMet = typeClazz.getMethod(fieldSetName, field
                            .getType());
                    switch (type.getSimpleName()) {
                        case "String":
                            fieldSetMet.invoke(instance, resultSet1.getString(field.getName()));
                            break;
                        case "Integer":
                            fieldSetMet.invoke(instance, resultSet1.getInt(field.getName()));
                            break;
                        default:
                            throw new RuntimeException();
                    }
                }
                list.add(instance);
            }
            return list;
        } catch (HoloClientException | InterruptedException | ExecutionException | SQLException e) {
            log.error(e.getMessage(), e);
            log.error("holo exec sql error: {}", e.getMessage());
            throw new RuntimeException("holo exec sql error: " + e.getMessage(), e);
        }
    }
    private static final Map<Class<?>, Set<String>> map = new HashMap<>();

    private static Set<String> getFieldSet(ResultSet resultSet, Class<?> typeClazz) {
        if (!map.containsKey(typeClazz)) {
            synchronized (HoloUtils.class) {
                if (!map.containsKey(typeClazz)) {
                    try {
                        ResultSetMetaData metaData = resultSet.getMetaData();
                        int columnCount = metaData.getColumnCount();
                        HashSet<String> set = new HashSet<>();
                        for (int i = 1; i <= columnCount; i++) {
                            String columnName = metaData.getColumnName(i);
                            set.add(columnName);
                        }
                        map.put(typeClazz, set);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                        e.printStackTrace();
                        throw new RuntimeException(e);
                    }
                }
            }
        }
        return map.get(typeClazz);
    }

}
