package com.winhc.task.util;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Stream;

/**
 * @author ZhangJi
 * @since 2021-10-29 15:56
 */
public class StreamEnhance {
    private static final Logger LOG = LoggerFactory.getLogger(StreamEnhance.class);

    public static <T> void parForEach(Stream<T> stream, Consumer<T> consumer, ExecutorService es) {
        try {
            es.submit(() -> stream.parallel().forEach(consumer)).get();
        } catch (InterruptedException | ExecutionException e) {
            LOG.error("parallel stream failed", e);
        }
    }

    public static <T> Stream<T> append(Stream<T> source, T first) {
        return Stream.concat(source, Stream.of(first));
    }

    public static <T> Stream<T> append(Stream<T> source, Collection<T> elements) {
        return Stream.concat(source, elements.stream());
    }

    public static <T> Stream<T> append(Stream<T> source, Stream<T> other) {
        return Stream.concat(source, other);
    }

    public static <T> Predicate<T> distinctByKey(
            Function<? super T, ?> keyExtractor) {

        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }

    public static <T> Stream<T> distinctBy(Stream<T> source, Function<? super T, ?> keyExtractor) {
        return source.filter(distinctByKey(keyExtractor));
    }

}
