package com.winhc.task.util;

import org.apache.commons.codec.binary.Hex;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;


/**
 * AES加密模式：ECB
 * 填充：pkcs7（java中pkcs5等效果于pkcs7）
 * 数据块：128位（16字节）
 * 密码(key)【128/192/256比特，16/24/32字符】：winhc_168_000000
 * 偏移量(iv)【ecb模式下不用iv】：
 * 输出：hex
 * 字符集：utf-8
 */

/**
 * @author: XuJiakai
 * 2021/2/3 11:44
 */
public class WinhcCryptoNative {

    private static final String key = "winhc_168_000000";


    private static final String CIPHER_ALGORITHM = "AES/ECB/PKCS5Padding";
    private final Lock lock = new ReentrantLock();

    private final Cipher cipher;
    private final SecretKey secretKey;


    public WinhcCryptoNative() {
        try {
            cipher = Cipher.getInstance(CIPHER_ALGORITHM);
            secretKey = new SecretKeySpec(key.getBytes(StandardCharsets.UTF_8), "AES");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private byte[] encrypt(byte[] data) {
        lock.lock();
        try {
            cipher.init(Cipher.ENCRYPT_MODE, secretKey);
            return cipher.doFinal(data);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
    }

    private byte[] decrypt(byte[] bytes) {
        lock.lock();
        try {
            cipher.init(Cipher.DECRYPT_MODE, secretKey);
            return cipher.doFinal(bytes);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
    }

    public String encryptHex(String data) {
        return Hex.encodeHexString(encrypt(data.getBytes(StandardCharsets.UTF_8)));
    }

    public String decryptStr(String data) {
        try {
            byte[] bytes = Hex.decodeHex(data);
            return new String(decrypt(bytes), StandardCharsets.UTF_8);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }


    public static void main(String[] args) {
        WinhcCryptoNative winhcCryptoNative = new WinhcCryptoNative();

        List<String> list = Arrays.asList("413026199901011010", "413026199901011012");
        for (String content : list) {
            String cipher = winhcCryptoNative.encryptHex(content);
            System.out.println(cipher);

            String plain = winhcCryptoNative.decryptStr(cipher);
            System.out.println(plain);
        }

    }
}
